#!/bin/bash
# vim: set ts=4 sw=4 sts noet mouse=a fdm=indent:

DEFAULT_DEST="${HOME}/.local/share"
DEFAULT_BIN="${HOME}/.local/bin"
BYPASS="n"
DEBUG="n"

SCRIPT_NAME=$(basename "$0")
PYTHON=python3
VERSION=$(cat ./msspec/version.py|cut -d\" -f2)
TIMEOUT=5
DEST_ALREADY_EXISTS="n"
ECHO="/bin/echo -e"
DATE=$(date +%Y%m%d_%H%M%S)
LOGFILE="${HOME}/msspec_install_${DATE}.log"
GETOUTFILE=$(mktemp)
LOCALBIN=$HOME/.local/bin
LINUX_OS="other_linux"


trap "abort_install" 2
trap "error_exit" 1 3 4 6

init_install () {
	${ECHO} "Installation started on $(date)" > ${LOGFILE}
	${ECHO} "0" > ${GETOUTFILE}
	get_os

	if [ "$BYPASS" = "n" ]; then
		${ECHO} ""
		${ECHO} "This program will install the MsSpec distribution on your computer"
		${ECHO} -n "Press C^c to cancel now...  "
		for i in $(seq -s" " 0 $TIMEOUT); do
		    ${ECHO} -n "\b";
		    ${ECHO} -n $(( $TIMEOUT-i ));
		    sleep 1;
		done
		${ECHO} ""
	fi

}

abort_install () {
    ${ECHO} "Installation aborted by the user" >> ${LOGFILE}
    error_exit
}

log_message () {
	printf "$1\n" >> ${LOGFILE}
}

log_message2 () {
	# echo to screen and to file
	printf "$1\n"
	log_message "$1\n"
}

cleanup () {
	stty echo
	rm -rf ${GETOUTFILE}
}

error_exit () {
	# An error occured
	printf "\n"
	printf " /!\  An error occured during the installation process.\n"
	printf "      Please see the $(basename ${LOGFILE}) file in your HOME folder\n"
	printf "      for more informations.\n"
	printf "      Below is an excerpt of the last 10 lines:\n"
	printf "\n"
	tail -v -n 10 "${LOGFILE}"

	# uninstall if needed
	pip uninstall -y msspec

	# if the DEST_FOLDER existed before, do NOT delete it
	if [ x"$DEST_ALREADY_EXISTS" = "xn" ]; then
		rm -rf ${DEST_FOLDER} 2>/dev/null
	fi
	cleanup && exit 1
}



get_os () {
	for os in "ubuntu" "mageia" "archlinux"; do
		cat /etc/*release /etc/*version /etc/*issue 2>/dev/null | grep -i "$os" 2>/dev/null 1>/dev/null
		if test $? -eq 0; then
			LINUX_OS="$os"
			break
		fi
	done
		
}


read_yes_no () {
    DEFAULT="$2"
    PROMPT="$1 (y/n) [${DEFAULT}]? "

    ANSWER=""
    RESPONSE=""

    VALID=1
    while test $VALID -ne 0; do
		printf "${PROMPT}"
        if [ x"$BYPASS" = "xn" ]; then 
			read "RESPONSE"
		else
			printf "\n"
		fi
        ANSWER="${RESPONSE:-${DEFAULT}}"
        case "${ANSWER}" in
            y|n) VALID=0 ;;
            *) printf "Invalid choice, please answer \"y\" or \"n\".\n"; VALID=1 ;;
        esac
    done
	log_message "QUESTION: $PROMPT"
	log_message "ANSWER  : $ANSWER"
}


read_folder () {
    DEFAULT="$2"
    PROMPT="$1 [${DEFAULT}]: "

	ANSWER=""
	unset FOLDER
	printf "${PROMPT}"
	if [ x"$BYPASS" = "xn" ]; then 
		read "ANSWER"
	else
		printf "\n"
	fi
	export FOLDER="${ANSWER:-$DEFAULT}"
	log_message "QUESTION: $PROMPT"
	log_message "ANSWER  : $FOLDER"
}



create_folder () {
    FOLDER="$1"

    ERR_MKDIR=1
    ERR_ABORT=2
    ERR_PERMS=3

    if ! test -d "${FOLDER}" ; then
        read_yes_no "The folder \"${FOLDER}\" does not exist. Should I create it" "y" #"$BYPASS"

        case "${ANSWER}" in
            y) mkdir -p "${FOLDER}" 2>>${LOGFILE} 1>>${LOGFILE} || return $ERR_MKDIR;;
            n) log_message2 "Installation aborted by the user"  && return $ERR_ABORT ;;
        esac
    fi

    if ! test -w "${FOLDER}" ; then
        log_message2 "You do not have permission to write into \"${FOLDER}\"."
        return $ERR_PERMS
    fi
}


wrap () {
	log_message "================================================================================"
	log_message "$2"
	log_message "================================================================================"
	if [ "$DEBUG" = "y" ]; then
		eval "($1) 2>&1" | tee -a ${LOGFILE}
		echo ${PIPESTATUS[0]} >${GETOUTFILE}
		
		rc=$(cat $GETOUTFILE)
		if [ $rc != 0 ]; then
			error_exit
		fi
	else
		eval "($1) 2>>${LOGFILE} 1>>${LOGFILE}" || echo $? >${GETOUTFILE} &
		patience "$2"
	fi
}

update_path () {
	# check if the shell path contains $BIN_FOLDER
	SHELLRC="$HOME/.$(basename $SHELL)rc"
	${ECHO} "$PATH" | grep "${BIN_FOLDER}" 2>/dev/null 1>/dev/null
	if test $? -ne 0; then
		${ECHO} "" >> $SHELLRC
		${ECHO} "# Add the user's local binary folder" >> $SHELLRC
		${ECHO} "export PATH=${BIN_FOLDER}:\$PATH" >> $SHELLRC
		${ECHO} "Your PATH variable has been updated, the folder"
		${ECHO} "${BIN_FOLDER} has been added to it."
		${ECHO} ""
		${ECHO} "Please source again your shell configuration file by typing:"
		${ECHO} "source $SHELLRC"
	fi	
}


success_message () {
	log_message "========================================================"
	log_message "MsSpec was successfully installed."
	log_message "========================================================"
}




patience () {
    MSG=$1
    PID=$!
    i=0
    ${ECHO} -n "$MSG...  "
    while ps -p $PID > /dev/null
    do
        sleep 0.1
        case $i in
            0) ${ECHO} -n "\b\b- " ;;
            1) ${ECHO} -n "\b\b\\ " ;;
            2) ${ECHO} -n "\b\b| " ;;
            3) ${ECHO} -n "\b\b/ "; i=-1 ;;
        esac
        i=$((i+1))
    done
	if test $(cat ${GETOUTFILE}) -ne 0; then
    	${ECHO} "\b\b Aborted!"
		error_exit
	fi
    ${ECHO} "\b\b Done."
}

show_help () {        
    echo "Usage: $SCRIPT_NAME [OPTIONS]"
    echo "List of possible options:"
    echo "    -p FOLDER   Default FOLDER for installation."
    echo "    -b FOLDER   Default FOLDER for binaries."
    echo "    -y          Accept all default choices."
    echo "    -d          Debug mode."
    echo "    -h          Show this message."
	exit 0
}


check_dependencies () {
	# we need make
	log_message "Checking if make is installed..."
	command -V make || return 1
	log_message "Ok\n"

	# we need gfortran
    gcc_ver_min=7
	log_message "Checking if gfortran >= $gcc_ver_min is installed..."
	command -V gfortran || return 1
	gcc_ver=$(gcc -dumpversion)
	echo "You have gfortran version $gcc_ver installed"
    test $gcc_ver -ge $gcc_ver_min || return 1
	log_message "Ok\n"

	# we need libcairo and dev files
	log_message "Checking if libcairo is installed..."
	pkg-config --exists cairo || return 1
	log_message "Ok\n"

	# we need Python 3
	py_ver_major_min=3
	py_ver_minor_min=0
	log_message "Checking if Python >= ${py_ver_major_min}.${py_ver_minor_min}..."
	message="You need Python version ${py_ver_major_min}.${py_ver_minor_min}"
    python --version|cut -d" " -f2 || return 1
    py_ver=$(python --version | cut -d" " -f2)
	echo "You have Python version $py_ver installed"
	py_ver_major=$(echo $py_ver|cut -d. -f1)
	py_ver_minor=$(echo $py_ver|cut -d. -f2)
	test $py_ver_major -ge $py_ver_major_min || return 1
	test $py_ver_minor -ge $py_ver_minor_min || return 1
	log_message "Ok\n"

	# we need virtualenv
	log_message "Checking if virtualenv is installed..."
	command -V virtualenv || return 1
	log_message "Ok\n"
	
}

local_install () {
	# Initialize the install process
	init_install

	# check dependencies
	wrap "check_dependencies" "Checking dependencies"

	# ask for a place to put the binary
	#read_folder "Please, type in the folder where to place the binary" "${DEFAULT_BIN}"
	#BIN_FOLDER=${FOLDER}
	#create_folder "${BIN_FOLDER}" || error_exit
	#export BIN_FOLDER

	# If in a virtualenv, ask if we should install inside
	if [ x"$VIRTUAL_ENV" != x ]; then
		read_yes_no "You are running a Python virtual environment. Do you wish to install msspec inside ?" "y"
		case ${ANSWER} in
			y)	# get the python version of this venv...
				v=$(python -V 2>&1 | cut -d" " -f2 | cut -d. -f1)
				# and stops if < 3
				if [ $v -lt 3 ]; then
					log_message "The Python version of this venv is < 3"
					error_exit
				fi
				;;
		   	n)	# The user decided to stop there
				log_message "Please deactivate your virtual environment and restart the setup program."
				error_exit
				;;
		esac
	# Else, we are not in a virtualenv
	else
		pip_opt=""
		# so ask if we should create one
		read_yes_no "Do you wish to create a virtual environment to install MsSpec inside" "y"
		case ${ANSWER} in
			y) 	# Create the destination folder
				read_folder "Please, type in the base installation folder" "${DEFAULT_DEST}"
				DEST_FOLDER=${FOLDER}/MsSpec-${VERSION}
				if [ -d $DEST_FOLDER ]; then 
					export DEST_ALREADY_EXISTS="y" # to avoid cleaning the existing DEST_FOLDER in case of any problem
					read_yes_no "The folder $DEST_FOLDER already exists. Overwrite" "y"
					case ${ANSWER} in
						y) 	# overwrite
							create_folder "${DEST_FOLDER}" || error_exit
							;;
						n)	# stop here otherwise
							error_exit
							;;
					esac
				fi
				export DEST_FOLDER
				wrap "virtualenv -vvv --python=${PYTHON} --system-site-packages --prompt=\"(msspec-${VERSION}) \" ${DEST_FOLDER}/venv" \
					"Create a Python virtual environment"
				. ${DEST_FOLDER}/venv/bin/activate
				;;
			n) 	# install in the user $HOME
				pip_opt="--user"
				;;
		esac
	fi

	# build and run setuptools to create a source distribution
	wrap "make sdist" \
	     "Building MsSpec python package"
	
	# install the package with pip
	wrap "pip install $pip_opt dist/msspec-*.tar.gz" \
	     "Installing pymsspec python package and its dependencies"

#
## move the frontend to the binary folder
##wrap "mkdir -p ${LOCALBIN} && \
##      cp ./msspec ${LOCALBIN} && chmod u+x ${LOCALBIN}/msspec && \
##	  ${ECHO} \"msspec installed in ${LOCALBIN}\"" \
##	 "Installing the msspec frontend"
# "cp ./msspec ${BIN_FOLDER} && chmod ugo+rx ${BIN_FOLDER}/msspec" "Installing the msspec frontend"
#
## move the uninstal script
#wrap "cp ./uninstall.sh ${DEST}" "Moving the uninstall script"
#
	# Run unit tests ?
	read_yes_no "Do you wish to run unit tests (${LOGFILE})" "y"
	if [ x"${ANSWER}" = "xy" ]; then
    	wrap "python -c \"from msspec.tests import run_tests; run_tests()\"" \
             "Runing test suite"
	fi


	# installation was a succes so ask if we keep the log file
	read_yes_no "Do you wish to keep the log file (${LOGFILE})" "n"
	if [ x"${ANSWER}" = "xn" ]; then
		rm -f "${LOGFILE}"
	fi
	
	# self-explanatory
	success_message
}



while getopts "p:b:yd" option
do
    case $option in
        p) export DEFAULT_DEST="$OPTARG"
           ;;
        b) export DEFAULT_BIN="$OPTARG"
           ;;
        y) export BYPASS="y"
           ;;
        d) export DEBUG="y"
           ;;
        h) show_help
           ;;
        *) show_help
           ;;
    esac
done




local_install
